@@ -62,6 +62,10 @@ class BaseConfig(object):
         "ELMD_REGTEST_DATA_DIR", "/tmp/specter_elm_regtest_plain_datadir"
     )
 
+    # the default timeout for Bitcoin/Liquid RPC-calls
+    BITCOIN_RPC_TIMEOUT = int(os.getenv("BITCOIN_RPC_TIMEOUT", "10"))
+    LIQUID_RPC_TIMEOUT = int(os.getenv("LIQUID_RPC_TIMEOUT", "10"))
+
     # The self-signed ssl-certificate which is lazily created is configurable to a certain extent
     SPECTER_SSL_CERT_SUBJECT_C = os.getenv("SPECTER_SSL_CERT_SUBJECT_C", "DE")
     SPECTER_SSL_CERT_SUBJECT_ST = os.getenv("SPECTER_SSL_CERT_SUBJECT_ST", "BDW")
@@ -104,6 +108,10 @@ class BaseConfig(object):
             "url": "",
         },
     }
+    # in seconds, tor is slow, so let's choose 10 seconds
+    FEEESTIMATION_REQUEST_TIMEOUT = int(
+        os.getenv("FEEESTIMATION_REQUEST_TIMEOUT", "10")
+    )
 
     # Babel integration. English listed first; other alphabetical by language code
     LANGUAGES = {
@@ -148,6 +156,10 @@ class TestConfig(BaseConfig):
     # API active by default in test-mode
     SPECTER_API_ACTIVE = _get_bool_env_var("SPECTER_API_ACTIVE", "True")
 
+    # See #1316 since Bitcoin v0.21.1 (not only) the importmulti-call takes longer than 10 seconds on cirrus
+    BITCOIN_RPC_TIMEOUT = 60
+    LIQUID_RPC_TIMEOUT = 60
+
 
 class CypressTestConfig(TestConfig):
     SPECTER_DATA_FOLDER = os.path.expanduser(
@@ -165,7 +177,13 @@ class CypressTestConfig(TestConfig):
     BTCD_REGTEST_DATA_DIR = os.getenv(
         "BTCD_REGTEST_DATA_DIR", "/tmp/specter_cypress_elm_regtest_plain_datadir"
     )
+    BITCOIN_RPC_TIMEOUT = 30
+    LIQUID_RPC_TIMEOUT = 40
 
 
 class ProductionConfig(BaseConfig):
     SECRET_KEY = secrets.token_urlsafe(16)
+    # There are some really slow machines out there. Creating a 2/4 multisig on an older MacBookAir
+    # Take already >30secs
+    BITCOIN_RPC_TIMEOUT = 60
+    LIQUID_RPC_TIMEOUT = 120
