@@ -3,15 +3,17 @@ from embit.ec import PrivateKey, PublicKey
 from embit.hashes import tagged_hash
 from embit.descriptor.checksum import add_checksum
 from embit.liquid.descriptor import LDescriptor
-from embit.liquid.pset import PSET
+from embit.liquid.pset import PSET, PSBTError
 from embit.liquid.addresses import addr_decode
 from embit.liquid.addresses import address as liquid_address
 from embit.liquid.networks import get_network
 from embit.liquid.transaction import LTransaction, unblind
+from embit.liquid import finalizer
 from embit.liquid import slip77
 from embit.psbt import read_string
-
+import copy
 from io import BytesIO
+from .util.pset import to_canonical_pset
 
 import logging
 
@@ -33,6 +35,43 @@ class LiquidRPC(BitcoinRPC):
       - extra data that should be encoded in the change rangeproof
     """
 
+    _master_blinding_key = None
+    _asset_labels = None
+
+    @property
+    def asset_labels(self):
+        if self._asset_labels is None:
+            self._asset_labels = super().__getattr__("dumpassetlabels")()
+        return self._asset_labels
+
+    @property
+    def master_blinding_key(self):
+        if self._master_blinding_key is None:
+            self._master_blinding_key = PrivateKey(
+                bytes.fromhex(self.dumpmasterblindingkey())
+            )
+        return self._master_blinding_key
+
+    def dumpassetlabels(self, *args, **kwargs):
+        # inject cache
+        res = super().__getattr__("dumpassetlabels")()
+        self._asset_labels = res
+        return res
+
+    def importmasterblindingkey(self, *args, **kwargs):
+        # clear cache
+        self._master_blinding_key = None
+        return super().__getattr__("importmasterblindingkey")(*args, **kwargs)
+
+    def _patch_assetlabels(self, obj):
+        # check if assets are labeled and replace with real assetsids
+        if isinstance(obj, dict):
+            for asset in list(obj.keys()):
+                if asset in self.asset_labels and asset != "bitcoin":
+                    assetid = self.asset_labels[asset]
+                    obj[assetid] = obj[asset]
+                    del obj[asset]
+
     def getbalance(
         self,
         dummy="*",
@@ -50,7 +89,9 @@ class LiquidRPC(BitcoinRPC):
         # if assetlabel is explicitly set to None - return all assets
         if assetlabel is not None:
             args.append(assetlabel)
-        return super().__getattr__("getbalance")(*args, **kwargs)
+        res = super().__getattr__("getbalance")(*args, **kwargs)
+        self._patch_assetlabels(res)
+        return res
 
     def _patch_descriptors(self, arr):
         # used by importmulti and importdescriptors
@@ -92,6 +133,9 @@ class LiquidRPC(BitcoinRPC):
         res = super().__getattr__("getbalances")(**kwargs)
         # if assetlabel is explicitly set to None - return as is
         if assetlabel is None:
+            for k in res:
+                for kk in res[k]:
+                    self._patch_assetlabels(res[k][kk])
             return res
         # otherwise get balances for a particular assetlabel
         for k in res:
@@ -100,18 +144,15 @@ class LiquidRPC(BitcoinRPC):
                 res[k][kk] = v
         return res
 
-    def decodepsbt(self, *args, **kwargs):
-        res = super().__getattr__("decodepsbt")(*args, **kwargs)
-        res["fee"] = res["fees"]["bitcoin"]
-        return res
-
     def getreceivedbyaddress(self, address, minconf=1, assetlabel="bitcoin", **kwargs):
         args = [address, minconf]
         if assetlabel is not None:
             args.append(assetlabel)
         return super().__getattr__("getreceivedbyaddress")(*args, **kwargs)
 
-    def walletcreatefundedpsbt(self, inputs, outputs, *args, blind=True, **kwargs):
+    def walletcreatefundedpsbt(
+        self, inputs, outputs, locktime=0, options={}, *args, blind=True, **kwargs
+    ):
         """
         Creates and blinds an Elements PSBT transaction.
         Arguments:
@@ -119,41 +160,110 @@ class LiquidRPC(BitcoinRPC):
         2. outputs: [{address: amount, "asset": asset}, ...] # TODO: add assets support
         3. locktime = 0
         4. options {includeWatching, changeAddress, subtractFeeFromOutputs,
-                    replaceable, add_inputs, feeRate, fee_rate}
+                    replaceable, add_inputs, feeRate, fee_rate, changeAddresses}
         5. bip32 derivations
         6. solving data
         7. blind = True - Specter-LiquidRPC specific thing - blind transaction after creation
         """
-        res = super().__getattr__("walletcreatefundedpsbt")(
-            inputs, outputs, *args, **kwargs
+        options = copy.deepcopy(options)
+        change_addresses = (
+            options.pop("changeAddresses") if "changeAddresses" in options else None
         )
+        destinations = []
+        for o in outputs:
+            for k in o:
+                if k != "asset":
+                    destinations.append(addr_decode(k)[0])
+        res = super().__getattr__("walletcreatefundedpsbt")(
+            inputs, outputs, locktime, options, *args, **kwargs
+        )
+        psbt = res.get("psbt", None)
+
+        # remove zero-output (bug in Elements)
+        # TODO: remove after release
+        if psbt:
+            try:
+                tx = PSET.from_string(psbt)
+                # check if there are zero outputs
+                has_zero = len([out for out in tx.outputs if out.value == 0]) > 0
+                has_blinded = any([out.blinding_pubkey for out in tx.outputs])
+                logger.error(has_zer, has_blinded)
+                if has_blinded and has_zero:
+                    tx.outputs = [out for out in tx.outputs if out.value > 0]
+                    psbt = str(tx)
+                    res["psbt"] = psbt
+            except:
+                pass
+
+        # replace change addresses from the transactions if we can
+        if change_addresses and psbt:
+            try:
+                tx = PSET.from_string(psbt)
+                cur = 0
+                for out in tx.outputs:
+                    # fee
+                    if out.script_pubkey.data == b"":
+                        continue
+                    # not change for sure
+                    if not out.bip32_derivations:
+                        continue
+                    # do change replacement
+                    if out.script_pubkey not in destinations:
+                        sc, bkey = addr_decode(change_addresses[cur])
+                        cur += 1
+                        out.script_pubkey = sc
+                        out.blinding_pubkey = bkey.sec() if bkey else None
+                        out.bip32_derivations = {}
+                        out.redeem_script = None
+                        out.witness_script = None
+                # fill derivation info
+                patched = (
+                    super().__getattr__("walletprocesspsbt")(str(tx), False).get("psbt")
+                )
+                patchedtx = PSET.from_string(patched)
+                assert len(tx.outputs) == len(patchedtx.outputs)
+                for out1, out2 in zip(tx.outputs, patchedtx.outputs):
+                    # fee
+                    if out1.script_pubkey.data == b"":
+                        continue
+                    # not change for sure
+                    if not out2.bip32_derivations:
+                        continue
+                    # do change replacement
+                    if out1.script_pubkey not in destinations:
+                        out1.bip32_derivations = out2.bip32_derivations
+                        out1.redeem_script = out2.redeem_script
+                        out1.witness_script = out2.witness_script
+
+                res["psbt"] = str(tx)
+            except Exception as e:
+                logger.error(e)
+                raise e
+
         psbt = res.get("psbt", None)
         # check if we should blind the transaction
         if psbt and blind:
             # check that change is also blinded - fixes a bug in pset branch
             tx = PSET.from_string(psbt)
-            der = None
             changepos = res.get("changepos", None)
-            if changepos is not None and len(args) >= 2:
-                addr = args[1].get("changeAddress", None)
-                if addr:
-                    _, bpub = addr_decode(addr)
-                    der = tx.outputs[changepos].bip32_derivations
-                    if bpub and (tx.outputs[changepos].blinding_pubkey is None):
-                        tx.outputs[changepos].blinding_pubkey = bpub.sec()
-                    res["psbt"] = str(tx)
-                    psbt = str(tx)
+            # no change output
+            if changepos < 0:
+                changepos = None
 
             # generate all blinding stuff ourselves in deterministic way
-            bpk = bytes.fromhex(self.dumpmasterblindingkey())
-            tx.unblind(PrivateKey(bpk))  # get values and blinding factors for inputs
-            seed = tagged_hash("liquid/blinding_seed", bpk)
-            tx.blind(seed)  # generate all blinding factors etc
-            # proprietary fields for Specter - 00 is global blinding seed
-            tx.unknown[b"\xfc\x07specter\x00"] = seed
+            tx.unblind(
+                self.master_blinding_key
+            )  # get values and blinding factors for inputs
+            seed = tagged_hash("liquid/blinding_seed", self.master_blinding_key.secret)
+            try:
+                tx.blind(seed)  # generate all blinding factors etc
+                # proprietary fields for Specter - 00 is global blinding seed
+                tx.unknown[b"\xfc\x07specter\x00"] = seed
+            except PSBTError:
+                seed = None
 
             # reblind and encode nonces in change output
-            if changepos is not None:
+            if seed and changepos is not None:
                 txseed = tx.txseed(seed)
                 # blinding seed to calculate per-output nonces
                 message = b"\x01\x00\x20" + txseed
@@ -176,34 +286,26 @@ class LiquidRPC(BitcoinRPC):
             res["psbt"] = str(tx)
         return res
 
-    def _cleanpsbt(self, psbt):
-        """Removes stuff that Core doesn't like"""
-        tx = PSET.from_string(psbt)
-        for inp in tx.inputs:
-            inp.value = None
-            inp.asset = None
-            inp.value_blinding_factor = None
-            inp.asset_blinding_factor = None
-
-        for out in tx.outputs:
-            if out.is_blinded:
-                out.asset = None
-                out.asset_blinding_factor = None
-                out.value = None
-                out.value_blinding_factor = None
-        return str(tx)
-
     def walletprocesspsbt(self, psbt, *args, **kwargs):
         try:
             if self.getwalletinfo().get("private_keys_enabled", False):
-                psbt = self._cleanpsbt(psbt)
+                psbt = to_canonical_pset(psbt)
         except Exception as e:
             logger.warn(f"Failed to clean psbt: {e}")
         return super().__getattr__("walletprocesspsbt")(psbt, *args, **kwargs)
 
     def finalizepsbt(self, psbt, *args, **kwargs):
-        psbt = self._cleanpsbt(psbt)
-        return super().__getattr__("finalizepsbt")(psbt, *args, **kwargs)
+        psbt = to_canonical_pset(psbt)
+        res = super().__getattr__("finalizepsbt")(psbt, *args, **kwargs)
+        if res["complete"] == False:
+            try:
+                # try using our finalizer
+                tx = finalizer.finalize_psbt(PSET.from_string(psbt))
+                if tx and self.testmempoolaccept([str(tx)]):
+                    return {"complete": True, "hex": str(tx)}
+            except Exception as e:
+                logger.exception(e)
+        return res
 
     def combinepsbt(self, psbts, *args, **kwargs):
         if len(psbts) == 0:
@@ -217,9 +319,7 @@ class LiquidRPC(BitcoinRPC):
             tx.xpubs.update(t2.xpubs)
             tx.unknown.update(t2.unknown)
 
-            for i in range(len(tx.inputs)):
-                inp1 = tx.inputs[i]
-                inp2 = t2.inputs[i]
+            for inp1, inp2 in zip(tx.inputs, t2.inputs):
                 inp1.value = inp1.value or inp2.value
                 inp1.value_blinding_factor = (
                     inp1.value_blinding_factor or inp2.value_blinding_factor
@@ -244,9 +344,7 @@ class LiquidRPC(BitcoinRPC):
                 inp1.unknown.update(inp2.unknown)
                 inp1.range_proof = inp1.range_proof or inp2.range_proof
 
-            for i in range(len(tx.outputs)):
-                out1 = tx.outputs[i]
-                out2 = t2.outputs[i]
+            for out1, out2 in zip(tx.outputs, t2.outputs):
                 out1.value_commitment = out1.value_commitment or out2.value_commitment
                 out1.value_blinding_factor = (
                     out1.value_blinding_factor or out2.value_blinding_factor
@@ -270,31 +368,27 @@ class LiquidRPC(BitcoinRPC):
                 out1.unknown.update(out2.unknown)
         return str(tx)
 
-    def decodepsbt(self, b64psbt, *args, **kwargs):
-        decoded = super().__getattr__("decodepsbt")(b64psbt, *args, **kwargs)
-        # pset branch - no fee and global tx fields...
-        if "tx" not in decoded or "fee" not in decoded:
-            pset = PSET.from_string(b64psbt)
-            if "tx" not in decoded:
-                decoded["tx"] = self.decoderawtransaction(str(pset.tx))
-            if "fee" not in decoded:
-                decoded["fee"] = pset.fee() * 1e-8
-        for out in decoded["outputs"]:
-            if "value" not in out:
-                out["value"] = -1
-        for out in decoded["tx"]["vout"]:
-            if "value" not in out:
-                out["value"] = -1
-        return decoded
-
     def decoderawtransaction(self, tx):
-        unblinded = self.unblindrawtransaction(tx)["hex"]
-        obj = super().__getattr__("decoderawtransaction")(unblinded)
+        blinded = super().__getattr__("decoderawtransaction")(tx)
+        try:
+            unblinded = self.unblindrawtransaction(tx)["hex"]
+            obj = super().__getattr__("decoderawtransaction")(unblinded)
+            if "vsize" in blinded:
+                obj["vsize"] = blinded["vsize"]
+            if "size" in blinded:
+                obj["size"] = blinded["size"]
+            if "weight" in blinded:
+                obj["weight"] = blinded["weight"]
+            if "hex" in blinded:
+                obj["hex"] = blinded["hex"]
+        except Exception as e:
+            logger.error(e)
+            obj = blinded
         try:
             # unblind the rest of outputs
             b = LTransaction.from_string(tx)
 
-            mbpk = PrivateKey(bytes.fromhex(self.dumpmasterblindingkey()))
+            mbpk = self.master_blinding_key
             net = get_network(self.getblockchaininfo().get("chain"))
 
             outputs = obj["vout"]
@@ -422,7 +516,7 @@ class LiquidRPC(BitcoinRPC):
             port=rpc.port,
             protocol=rpc.protocol,
             path=rpc.path,
-            timeout=rpc.timeout,
+            timeout=cls.default_timeout,  # Elements is slower
             session=rpc.session,
             proxy_url=rpc.proxy_url,
             only_tor=rpc.only_tor,
