@@ -9,12 +9,12 @@ import hashlib
 from collections import OrderedDict
 from io import BytesIO
 
-from ..helpers import alias, load_jsons, is_liquid
+from ..helpers import alias, load_jsons, is_liquid, add_dicts
 from ..persistence import delete_file, delete_folder
 from ..rpc import RpcError, get_default_datadir
 from ..specter_error import SpecterError
 from ..util.descriptor import AddChecksum
-from ..wallet import Wallet
+from ..wallet import Wallet, purposes
 from ..liquid.wallet import LWallet
 
 from embit import ec
@@ -34,27 +34,6 @@ from embit.descriptor.checksum import add_checksum
 
 logger = logging.getLogger(__name__)
 
-purposes = OrderedDict(
-    {
-        None: "General",
-        "wpkh": "Single (Segwit)",
-        "sh-wpkh": "Single (Nested)",
-        "pkh": "Single (Legacy)",
-        "wsh": "Multisig (Segwit)",
-        "sh-wsh": "Multisig (Nested)",
-        "sh": "Multisig (Legacy)",
-    }
-)
-
-addrtypes = {
-    "pkh": "legacy",
-    "sh-wpkh": "p2sh-segwit",
-    "wpkh": "bech32",
-    "sh": "legacy",
-    "sh-wsh": "p2sh-segwit",
-    "wsh": "bech32",
-}
-
 
 class WalletManager:
     # chain is required to manage wallets when bitcoind is not running
@@ -83,7 +62,7 @@ class WalletManager:
         self.WalletClass = LWallet if is_liquid(chain) else Wallet
         self.update(data_folder, rpc, chain)
 
-    def update(self, data_folder=None, rpc=None, chain=None):
+    def update(self, data_folder=None, rpc=None, chain=None, use_threading=True):
         if self.is_loading:
             return
         self.is_loading = True
@@ -95,7 +74,7 @@ class WalletManager:
                 data_folder = os.path.expanduser(data_folder)
             # creating folders if they don't exist
             if not os.path.isdir(data_folder):
-                os.mkdir(data_folder)
+                os.makedirs(data_folder, exist_ok=True)
         self.working_folder = None
         if self.chain is not None and self.data_folder is not None:
             self.working_folder = os.path.join(self.data_folder, self.chain)
@@ -123,7 +102,7 @@ class WalletManager:
             for k in list(self.wallets.keys()):
                 if k not in self.wallets_update_list:
                     self.wallets.pop(k)
-            if self.allow_threading:
+            if self.allow_threading and use_threading:
                 t = threading.Thread(
                     target=self._update,
                     args=(
@@ -182,16 +161,11 @@ class WalletManager:
                                 for psbt in loaded_wallet.pending_psbts:
                                     logger.info(
                                         "lock %s " % wallet_alias,
-                                        loaded_wallet.pending_psbts[psbt]["tx"]["vin"],
+                                        loaded_wallet.pending_psbts[psbt].utxo_dict(),
                                     )
                                     loaded_wallet.rpc.lockunspent(
                                         False,
-                                        [
-                                            utxo
-                                            for utxo in loaded_wallet.pending_psbts[
-                                                psbt
-                                            ]["tx"]["vin"]
-                                        ],
+                                        loaded_wallet.pending_psbts[psbt].utxo_dict(),
                                     )
                             if len(loaded_wallet.frozen_utxo) > 0:
                                 loaded_wallet.rpc.lockunspent(
@@ -306,7 +280,7 @@ class WalletManager:
             logger.debug(f"Updating WalletManager rpc {self._rpc} with None")
         self._rpc = value
 
-    def create_wallet(self, name, sigs_required, key_type, keys, devices):
+    def create_wallet(self, name, sigs_required, key_type, keys, devices, **kwargs):
         try:
             walletsindir = [
                 wallet["name"] for wallet in self.rpc.listwalletdir()["wallets"]
@@ -322,82 +296,20 @@ class WalletManager:
             wallet_alias = alias("%s %d" % (name, i))
             i += 1
 
-        arr = key_type.split("-")
-        descs = [key.metadata["combined"] for key in keys]
-        recv_descs = ["%s/0/*" % desc for desc in descs]
-        change_descs = ["%s/1/*" % desc for desc in descs]
-        if len(keys) > 1:
-            recv_descriptor = "sortedmulti({},{})".format(
-                sigs_required, ",".join(recv_descs)
-            )
-            change_descriptor = "sortedmulti({},{})".format(
-                sigs_required, ",".join(change_descs)
-            )
-        else:
-            recv_descriptor = recv_descs[0]
-            change_descriptor = change_descs[0]
-        for el in arr[::-1]:
-            recv_descriptor = "%s(%s)" % (el, recv_descriptor)
-            change_descriptor = "%s(%s)" % (el, change_descriptor)
-
-        if is_liquid(self.chain):
-            blinding_key = None
-            if len(devices) == 1:
-                blinding_key = devices[0].blinding_key
-            if not blinding_key:
-                desc = Descriptor.from_string(recv_descriptor.split("#")[0])
-                # For now we use sha256(b"blinding_key", xor(chaincodes)) as a blinding key
-                # where chaincodes are corresponding to xpub of the first receiving address
-                xor = bytearray(32)
-                desc_keys = desc.derive(0).keys
-                for k in desc_keys:
-                    if k.is_extended:
-                        chaincode = k.key.chain_code
-                        for i in range(32):
-                            xor[i] = xor[i] ^ chaincode[i]
-                secret = hashlib.sha256(b"blinding_key" + bytes(xor)).digest()
-                blinding_key = ec.PrivateKey(secret).wif()
-            if blinding_key:
-                recv_descriptor = f"blinded(slip77({blinding_key}),{recv_descriptor})"
-                change_descriptor = (
-                    f"blinded(slip77({blinding_key}),{change_descriptor})"
-                )
-
-        recv_descriptor = AddChecksum(recv_descriptor)
-        change_descriptor = AddChecksum(change_descriptor)
-
-        # v20.99 is pre-v21 Elements Core for descriptors
-        if self.bitcoin_core_version_raw >= 209900:
-            # Use descriptor wallet
-            self.rpc.createwallet(
-                os.path.join(self.rpc_path, wallet_alias), True, True, "", False, True
-            )
-        else:
-            self.rpc.createwallet(os.path.join(self.rpc_path, wallet_alias), True)
-
-        w = self.WalletClass(
-            name,
-            wallet_alias,
-            "{} of {} {}".format(sigs_required, len(keys), purposes[key_type])
-            if len(keys) > 1
-            else purposes[key_type],
-            addrtypes[key_type],
-            "",
-            -1,
-            "",
-            -1,
-            0,
-            0,
-            recv_descriptor,
-            change_descriptor,
-            keys,
-            devices,
-            sigs_required,
-            {},
-            [],
-            os.path.join(self.working_folder, "%s.json" % wallet_alias),
+        w = self.WalletClass.create(
+            self.rpc,
+            self.rpc_path,
+            self.working_folder,
             self.device_manager,
             self,
+            name,
+            wallet_alias,
+            sigs_required,
+            key_type,
+            keys,
+            devices,
+            self.bitcoin_core_version_raw,
+            **kwargs,
         )
         # save wallet file to disk
         if w and self.working_folder is not None:
@@ -405,6 +317,7 @@ class WalletManager:
         # get Wallet class instance
         if w:
             self.wallets[name] = w
+            logger.info(f"Successfully created Wallet {name}")
             return w
         else:
             raise ("Failed to create new wallet")
@@ -439,6 +352,17 @@ class WalletManager:
             wallet.save_to_file()
         self.update()
 
+    def joined_balance(self):
+        """
+        Joined balance of all wallets.
+        I don't call it full balance because
+        full balance is different - see wallet.fullbalance...
+        """
+        balance = {}
+        for wallet in self.wallets.values():
+            add_dicts(balance, wallet.balance)
+        return balance
+
     def full_txlist(
         self,
         fetch_transactions=True,
