@@ -1,7 +1,8 @@
 import logging
-import requests, json, os
+import requests, urllib3, json, os
 import os, sys, errno
 from .helpers import is_ip_private
+from .specter_error import SpecterError
 
 logger = logging.getLogger(__name__)
 
@@ -113,7 +114,7 @@ def get_configs(config=None, datadir=get_default_datadir()):
 
 
 def detect_rpc_confs(config=None, datadir=get_default_datadir()):
-    rpcconfs = get_configs(config)
+    rpcconfs = get_configs(config, datadir)
     rpc_arr = []
     for conf in rpcconfs:
         rpc_arr.append(conf)
@@ -189,6 +190,7 @@ class RpcError(Exception):
     try:
         rpc.does_not_exist()
     except RpcError as rpce:
+        assert rpce.status_code == 401 # A https-status-code
         assert rpce.error_code == -32601
         assert rpce.error_msg == "Method not found"
     See for error_codes https://github.com/bitcoin/bitcoin/blob/v0.15.0.1/src/rpc/protocol.h#L32L87
@@ -214,9 +216,14 @@ class RpcError(Exception):
 class BitcoinRPC:
     counter = 0
 
+    # These are used for tracing the calls without too many duplicates
     last_call_hash = None
     last_call_hash_counter = 0
 
+    # https://docs.python-requests.org/en/master/user/quickstart/#timeouts
+    # None means until connection closes. It's specified in seconds
+    default_timeout = None  # seconds
+
     def __init__(
         self,
         user="bitcoin",
@@ -238,7 +245,7 @@ class BitcoinRPC:
         self.protocol = protocol
         self.host = host
         self.path = path
-        self.timeout = timeout
+        self.timeout = timeout or self.__class__.default_timeout
         self.proxy_url = proxy_url
         self.only_tor = only_tor
         self.r = None
@@ -334,13 +341,46 @@ class BitcoinRPC:
         timeout = self.timeout
         if "timeout" in kwargs:
             timeout = kwargs["timeout"]
+
+        if kwargs.get("no_wait"):
+            # Zero is treated like None, i.e. infinite wait
+            timeout = 0.001
+
         url = self.url
         if "wallet" in kwargs:
             url = url + "/wallet/{}".format(kwargs["wallet"])
         self.trace_call(url, payload)
-        r = self.session.post(
-            url, data=json.dumps(payload), headers=headers, timeout=timeout
-        )
+        try:
+            r = self.session.post(
+                url, data=json.dumps(payload), headers=headers, timeout=timeout
+            )
+        except (requests.exceptions.Timeout, urllib3.exceptions.ReadTimeoutError) as to:
+            # Timeout is effectively one of the two:
+            # ConnectTimeout: The request timed out while trying to connect to the remote server
+            # ReadTimeout: The server did not send any data in the allotted amount of time.
+            # ReadTimeoutError: Raised when a socket timeout occurs while receiving data from a server
+            if kwargs.get("no_wait"):
+                # Used for rpc calls that don't immediately return (e.g. rescanblockchain) so we don't
+                # expect any data back anyway. __getattr__ expects a list of formatted json.
+                return [{"error": None, "result": None}]
+
+            logger.error(
+                "Timeout after {} secs while {} call({: <28}) payload:{} Exception: {}".format(
+                    timeout,
+                    self.__class__.__name__,
+                    "/".join(url.split("/")[3:]),
+                    payload,
+                    to,
+                )
+            )
+            raise SpecterError(
+                "Timeout after {} secs while {} call({: <28}). Check the logs for more details.".format(
+                    timeout,
+                    self.__class__.__name__,
+                    "/".join(url.split("/")[3:]),
+                    payload,
+                )
+            )
         self.r = r
         if r.status_code != 200:
             logger.debug(f"last call FAILED: {r.text} (raising RpcError)")
