@@ -9,6 +9,7 @@ from flask import (
     jsonify,
     flash,
 )
+from flask_babel import lazy_gettext as _
 from flask_login import login_required, current_user, logout_user
 from flask import current_app as app
 from ..helpers import alias
@@ -29,10 +30,12 @@ def login():
     if request.method == "POST":
         rate_limit()
         auth = app.specter.config["auth"]
+
         if auth["method"] == "none":
             app.login("admin")
-            app.logger.info("AUDIT: Successfull Login no credentials")
+            app.logger.info("AUDIT: Successful Login no credentials")
             return redirect_login(request)
+
         if auth["method"] == "rpcpasswordaspin":
             # TODO: check the password via RPC-call
             if (
@@ -40,13 +43,16 @@ def login():
                 or not app.specter.default_node.rpc.test_connection()
             ):
                 if app.specter.default_node.password == request.form["password"]:
-                    app.login("admin")
+                    app.login("admin", request.form["password"])
                     app.logger.info(
                         "AUDIT: Successfull Login via RPC-credentials (node disconnected)"
                     )
                     return redirect_login(request)
+
                 flash(
-                    "We could not check your password, maybe Bitcoin Core is not running or not configured?",
+                    _(
+                        "We could not check your password, maybe Bitcoin Core is not running or not configured?"
+                    ),
                     "error",
                 )
                 app.logger.info("AUDIT: Failed to check password")
@@ -61,25 +67,28 @@ def login():
             rpc = app.specter.default_node.rpc.clone()
             rpc.password = request.form["password"]
             if rpc.test_connection():
-                app.login("admin")
+                app.login("admin", request.form["password"])
                 app.logger.info("AUDIT: Successfull Login via RPC-credentials")
                 return redirect_login(request)
+
         elif auth["method"] == "passwordonly":
             password = request.form["password"]
-            if verify_password(app.specter.user_manager.admin.password, password):
-                app.login("admin")
+            if verify_password(app.specter.user_manager.admin.password_hash, password):
+                app.login("admin", request.form["password"])
                 return redirect_login(request)
+
         elif auth["method"] == "usernamepassword":
             # TODO: This way both "User" and "user" will pass as usernames, should there be strict check on that here? Or should we keep it like this?
             username = request.form["username"]
             password = request.form["password"]
             user = app.specter.user_manager.get_user_by_username(username)
             if user:
-                if verify_password(user.password, password):
-                    app.login(user.id)
+                if verify_password(user.password_hash, password):
+                    app.login(user.id, request.form["password"])
                     return redirect_login(request)
+
         # Either invalid method or incorrect credentials
-        flash("Invalid username or password", "error")
+        flash(_("Invalid username or password"), "error")
         app.logger.info("AUDIT: Invalid password login attempt")
         return (
             render_template(
@@ -108,14 +117,16 @@ def register():
         otp = request.form["otp"]
         if not username:
             flash(
-                "Please enter a username.",
+                _("Please enter a username."),
                 "error",
             )
             return redirect("register?otp={}".format(otp))
         min_chars = int(app.specter.config["auth"]["password_min_chars"])
         if not password or len(password) < min_chars:
             flash(
-                "Please enter a password of a least {} characters.".format(min_chars),
+                _("Please enter a password of a least {} characters.").format(
+                    min_chars
+                ),
                 "error",
             )
             return redirect("register?otp={}".format(otp))
@@ -126,25 +137,36 @@ def register():
                 i += 1
                 user_id = "{}{}".format(alias(username), i)
             if app.specter.user_manager.get_user_by_username(username):
-                flash("Username is already taken, please choose another one", "error")
+                flash(
+                    _("Username is already taken, please choose another one"), "error"
+                )
                 return redirect("register?otp={}".format(otp))
             app.specter.otp_manager.remove_new_user_otp(otp)
             config = {
                 "explorers": {"main": "", "test": "", "regtest": "", "signet": ""},
                 "hwi_bridge_url": "/hwi/api/",
             }
-            password_hash = hash_password(password)
-            user = User(user_id, username, password_hash, config, app.specter)
-            app.specter.user_manager.add_user(user)
+
+            user = app.specter.user_manager.create_user(
+                user_id=user_id,
+                username=username,
+                plaintext_password=password,
+                config=config,
+            )
+
             flash(
-                "You have registered successfully, \
+                _(
+                    "You have registered successfully, \
 please login with your new account to start using Specter"
+                )
             )
             return redirect(url_for("auth_endpoint.login"))
         else:
             flash(
-                "Invalid registration link, \
-please request a new link from the node operator.",
+                _(
+                    "Invalid registration link, \
+please request a new link from the node operator."
+                ),
                 "error",
             )
             return redirect("register?otp={}".format(otp))
@@ -154,13 +176,22 @@ please request a new link from the node operator.",
 @auth_endpoint.route("/logout", methods=["GET", "POST"])
 def logout():
     logout_user()
-    flash("You were logged out", "info")
+    if "timeout" in request.args:
+        flash(_("You were automatically logged out"), "info")
+    else:
+        flash(_("You were logged out"), "info")
     return redirect(url_for("auth_endpoint.login"))
 
 
 ################### Util ######################
 def redirect_login(request):
-    flash("Logged in successfully.", "info")
+    flash(_("Logged in successfully."), "info")
+
+    # If the user is auto-logged out, hide_sensitive_info will be set. If they're
+    #   explicitly logging in now, clear the setting and reveal user's info.
+    if app.specter.hide_sensitive_info:
+        app.specter.update_hide_sensitive_info(False, current_user)
+
     if request.form.get("next") and request.form.get("next") != "None":
         response = redirect(request.form["next"])
     else:
