@@ -1,16 +1,15 @@
-import random, traceback, socket, threading, os
-from datetime import datetime
+import random, traceback
 from binascii import unhexlify
 from flask import make_response
 from flask_wtf.csrf import CSRFError
 from werkzeug.exceptions import MethodNotAllowed
-from flask import render_template, request, redirect, url_for, flash, Markup
+from flask import render_template, request, redirect, url_for, flash
+from flask_babel import lazy_gettext as _
 from flask_login import login_required, current_user
 from ..helpers import (
     generate_mnemonic,
     notify_upgrade,
 )
-from ..specter import Specter
 from ..specter_error import SpecterError, ExtProcTimeoutException
 from pathlib import Path
 
@@ -27,6 +26,7 @@ app.register_blueprint(filters_bp)
 # Setup specter endpoints
 from .auth import auth_endpoint
 from .devices import devices_endpoint
+from .nodes import nodes_endpoint
 from .price import price_endpoint
 from .settings import settings_endpoint
 from .setup import setup_endpoint
@@ -35,6 +35,7 @@ from ..rpc import RpcError
 
 app.register_blueprint(auth_endpoint, url_prefix="/auth")
 app.register_blueprint(devices_endpoint, url_prefix="/devices")
+app.register_blueprint(nodes_endpoint, url_prefix="/nodes")
 app.register_blueprint(price_endpoint, url_prefix="/price")
 app.register_blueprint(settings_endpoint, url_prefix="/settings")
 app.register_blueprint(setup_endpoint, url_prefix="/setup")
@@ -48,10 +49,11 @@ def server_rpc_error(rpce):
     """Specific SpecterErrors get passed on to the User as flash"""
     if rpce.error_code == -18:  # RPC_WALLET_NOT_FOUND
         flash(
-            "Wallet not found. Specter reloaded all Wallets, please try again.", "error"
+            _("Wallet not found. Specter reloaded all wallets, please try again."),
+            "error",
         )
     else:
-        flash(f"BitcoinCore RpcError: {str(rpce)}", "error")
+        flash(_("Bitcoin Core RpcError: {}").format(str(rpce)), "error")
     try:
         app.specter.wallet_manager.update()
     except SpecterError as se:
@@ -61,13 +63,17 @@ def server_rpc_error(rpce):
 
 @app.errorhandler(SpecterError)
 def server_specter_error(se):
-    """Specific EpecterErrors get passed on to the User as flash"""
+    """Specific SpecterErrors get passed on to the User as flash"""
     flash(str(se), "error")
     try:
         app.specter.wallet_manager.update()
     except SpecterError as se:
         flash(str(se), "error")
-    return redirect(url_for("about"))
+    if request.method == "POST":
+        return redirect(request.url)
+    # potentially avoiding http loops. Might be improvable but how?
+    else:
+        return redirect(url_for("about"))
 
 
 @app.errorhandler(Exception)
@@ -92,10 +98,17 @@ def server_error_timeout(e):
         app.specter.check()
     app.logger.error("ExternalProcessTimeoutException: %s" % e)
     flash(
-        "Bitcoin Core is not coming up in time. Maybe it's just slow but please check the logs below",
+        _(
+            "Bitcoin Core is not coming up in time. Maybe it's just slow but please check the logs below"
+        ),
         "warn",
     )
-    return redirect(url_for("settings_endpoint.bitcoin_core_internal_logs"))
+    return redirect(
+        url_for(
+            "node_settings.bitcoin_core_internal_logs",
+            node_alias=app.specter.node.alias,
+        )
+    )
 
 
 @app.errorhandler(CSRFError)
@@ -106,7 +119,7 @@ def server_error_csrf(e):
     app.logger.error("CSRF Exception: %s" % e)
     trace = traceback.format_exc()
     app.logger.error(trace)
-    flash("Session expired. Please refresh and try again.", "error")
+    flash(_("Session expired. Please refresh and try again."), "error")
     return redirect(request.url)
 
 
@@ -116,7 +129,7 @@ def server_error_405(e):
     app.logger.error("405 MethodNotAllowed Exception: %s" % e)
     trace = traceback.format_exc()
     app.logger.error(trace)
-    flash("Session expired. Please refresh and try again.", "error")
+    flash(_("Session expired. Please refresh and try again."), "error")
     return redirect(request.url)
 
 
@@ -173,7 +186,12 @@ def about():
             app.specter.reset_setup("bitcoind")
             app.specter.reset_setup("torbrowser")
 
-    return render_template("base.jinja", specter=app.specter, rand=rand)
+    return render_template(
+        "base.jinja",
+        specter=app.specter,
+        rand=rand,
+        supported_languages=app.supported_languages,
+    )
 
 
 # TODO: Move all these below to REST API
@@ -199,7 +217,12 @@ def wallets_loading():
 @app.route("/generatemnemonic/", methods=["GET", "POST"])
 @login_required
 def generatemnemonic():
-    return {"mnemonic": generate_mnemonic(strength=int(request.form["strength"]))}
+    return {
+        "mnemonic": generate_mnemonic(
+            strength=int(request.form["strength"]),
+            language_code=app.get_language_code(),
+        )
+    }
 
 
 ################ RPC data utils ####################
@@ -231,6 +254,22 @@ def get_scantxoutset_status():
     }
 
 
+@app.route("/toggle_hide_sensitive_info/", methods=["POST"])
+@login_required
+@app.csrf.exempt  # might get called by a timeout in the browser --> csrf-issues
+def toggle_hide_sensitive_info():
+    try:
+        app.specter.update_hide_sensitive_info(
+            not app.specter.hide_sensitive_info, current_user
+        )
+        return {"success": True}
+    except Exception as e:
+        app.logger.warning(
+            "Failed to update sensitive info display settings. Exception: {}".format(e)
+        )
+    return {"success": False}
+
+
 @app.route("/bitcoin.pdf")
 @login_required
 def get_whitepaper():
@@ -278,5 +317,7 @@ def get_whitepaper():
     else:
         return render_template(
             "500.jinja",
-            error="You need a mainnet node to retrieve the whitepaper. Check your node configurations.",
+            error=_(
+                "You need a mainnet node to retrieve the whitepaper. Check your node configurations."
+            ),
         )
