@@ -19,7 +19,9 @@ from requests.exceptions import ConnectionError
 from stem.control import Controller
 from urllib3.exceptions import NewConnectionError
 
-from .helpers import clean_psbt, deep_update, is_liquid, is_testnet
+from cryptoadvance.specter.devices.device_types import DeviceTypes
+
+from .helpers import clean_psbt, deep_update, is_liquid, is_testnet, get_asset_label
 from .internal_node import InternalNode
 from .liquid.rpc import LiquidRPC
 from .managers.config_manager import ConfigManager
@@ -41,6 +43,7 @@ from .specter_error import ExtProcTimeoutException, SpecterError
 from .tor_daemon import TorDaemonController
 from .user import User
 from .util.checker import Checker
+from .util.version import VersionChecker
 from .util.price_providers import update_price
 from .util.setup_states import SETUP_STATES
 from .util.tor import get_tor_daemon_suffix
@@ -53,6 +56,7 @@ class Specter:
 
     # use this lock for all fs operations
     lock = threading.Lock()
+    _default_asset = None
 
     def __init__(self, data_folder="./data", config={}, internal_bitcoind_version=""):
         if data_folder.startswith("~"):
@@ -65,7 +69,14 @@ class Specter:
 
         self.data_folder = data_folder
 
-        self.user_manager = UserManager(self)
+        self.user_manager = UserManager(
+            self
+        )  # has to come before calling VersionChecker()
+
+        # version checker
+        # checks for new versions once per hour
+        self.version = VersionChecker(specter=self)
+        self.version.start()
 
         self._config_manager = ConfigManager(self.data_folder, config)
 
@@ -303,6 +314,14 @@ class Specter:
         """update the fee estimator option and its url if custom"""
         self.config_manager.update_fee_estimator(fee_estimator, custom_url, user)
 
+    # mark
+    def update_tor_type(self, tor_type, user):
+        """update the Tor proxy url"""
+        if tor_type == "builtin":
+            self.update_proxy_url("socks5h://localhost:9050", user)
+            self.update_tor_control_port("", user)
+        self.config_manager.update_tor_type(tor_type, user)
+
     # mark
     def update_proxy_url(self, proxy_url, user):
         """update the Tor proxy url"""
@@ -431,7 +450,10 @@ class Specter:
         return res
 
     def estimatesmartfee(self, blocks):
-        return self.rpc.estimatesmartfee(blocks)
+        res = self.rpc.estimatesmartfee(blocks)
+        if "feerate" not in res and self.is_liquid:
+            return 0.000001
+        return res
 
     @property
     def bitcoind_path(self):
@@ -457,6 +479,10 @@ class Specter:
     def bitcoin_core_version_raw(self):
         return self.node.bitcoin_core_version_raw
 
+    @property
+    def taproot_support(self):
+        return self.node.taproot_support
+
     @property
     def chain(self):
         return self.node.chain
@@ -489,9 +515,42 @@ class Specter:
     def explorer_id(self):
         return self.user_config.get("explorer_id", {}).get(self.chain, "CUSTOM")
 
+    @property
+    def asset_labels(self):
+        user_assets = self.user_config.get("asset_labels", {}).get(self.chain, {})
+        node_assets = self.node.asset_labels
+        asset_labels = {}
+        deep_update(asset_labels, node_assets)
+        deep_update(asset_labels, user_assets)
+        return asset_labels
+
+    @property
+    def default_asset(self):
+        """returns hash of LBTC"""
+        if self._default_asset is None:
+            for asset, lbl in self.asset_labels.items():
+                if lbl in ["LBTC", "tLBTC"]:
+                    self._default_asset = asset
+                    return asset
+        return self._default_asset
+
+    def asset_label(self, asset):
+        if asset == "":
+            return ""
+        return get_asset_label(asset, known_assets=self.asset_labels)
+
+    def update_asset_label(self, asset, label):
+        if asset == self.default_asset:
+            raise SpecterError("LBTC should stay LBTC")
+        self.config_manager.update_asset_label(asset, label, self.chain, self.user)
+
     @property
     def fee_estimator(self):
-        return self.user_config.get("fee_estimator", "mempool")
+        return self.user_config.get("fee_estimator", "bitcoin_core")
+
+    @property
+    def tor_type(self):
+        return self.user_config.get("tor_type", "builtin")
 
     @property
     def proxy_url(self):
@@ -567,6 +626,14 @@ class Specter:
     def hide_sensitive_info(self):
         return self.user_config.get("hide_sensitive_info", False)
 
+    @property
+    def autohide_sensitive_info_timeout(self):
+        return self.user_config.get("autohide_sensitive_info_timeout_minutes", 20)
+
+    @property
+    def autologout_timeout(self):
+        return self.user_config.get("autologout_timeout_hours", 4)
+
     def requests_session(self, force_tor=False):
         requests_session = requests.Session()
         if self.only_tor or force_tor:
@@ -600,8 +667,12 @@ class Specter:
                     data = zipfile.ZipInfo("{}.json".format(device.alias))
                     data.date_time = time.localtime(time.time())[:6]
                     data.compress_type = zipfile.ZIP_DEFLATED
+                    device = device.json
+                    # Exporting the bitcoincore hot wallet as watchonly
+                    if device["type"] == DeviceTypes.BITCOINCORE:
+                        device["type"] = DeviceTypes.BITCOINCORE_WATCHONLY
                     zf.writestr(
-                        "devices/{}.json".format(device.alias), json.dumps(device.json)
+                        "devices/{}.json".format(device["alias"]), json.dumps(device)
                     )
         memory_file.seek(0)
         return memory_file
